package org.expeditee.gui;

import java.util.Arrays;

import org.expeditee.items.Item;
import org.expeditee.settings.experimental.ExperimentalFeatures;


public class Help {
	
	// contexts
	public static final int background = 0, item = 1;
	
	// mouse buttons
	public static final int left = 1, middle = 2, right = 4;
	private static final String[] buttons = { "", "L", "M", "LM", "R", "LR", "MR" };
	
	// modifiers
	public static final int none = 0, control = 1, shift = 2, drag = 4, hold = 8;
	private static final String[] modifiers = { "", "^", "+", "^+", "d", "d^", "d+", "d^+" };
	
	// other
	public static final int panning = 1, action = 2;
	
	// separator between command and description
	private static final String separator = ": ";
	
	/**
	 * Set each index of $array to reference $value
	 */
	private static final void fill(String[] array, String value) {
		for(int i = 0; i < array.length; i++) {
			array[i] = value;
		}
	}
	
	/**
	 * Set each index of $array to reference an array filled with $value
	 */
	private static final void fill(String[][] array, String value) {
		array[0] = new String[array[0].length];
		fill(array[0], value);
   		for(int i = 1; i < array.length; i++) {
   			array[i] = array[0];
   		}
	}
	
	/**
	 * Set the indices of $array that have $indexBit set (or not set if $invert) to reference $value
	 */
	private static final void fill(String[] array, int indexBit, String value, boolean invert, boolean lazy) {
		for(int i = 0; i < array.length; i++) {
			if((!lazy && ((i & indexBit) == indexBit && !invert) || ((i & indexBit) == 0 && invert)) ||
				(lazy && ((i & indexBit) != 0 && !invert)        || ((i & indexBit) != indexBit && invert))){
				array[i] = value;
			}
		}
	}
	
	/**
	 * Calls fill(array, indexBit, value, false, false)
	 */
	private static final void fill(String[] array, int indexBit, String value) {
		fill(array, indexBit, value, false, false);
	}
	
	/**
	 * Set the indices of $array that have $indexBit set (or not set if $invert) to reference an array filled with $value
	 */
	private static final void fill(String[][] array, int indexBit, String value, boolean invert, boolean lazy) {
		for(int i = 0; i < array.length; i++) {
			if((!lazy && ((i & indexBit) == indexBit && !invert) || ((i & indexBit) == 0 && invert)) ||
				(lazy && ((i & indexBit) != 0 && !invert)        || ((i & indexBit) != indexBit && invert))){
				fill(array[i] = Arrays.copyOf(array[i], array[i].length), value);
			}
		}
	}
	
	/**
	 * Calls fill(array, indexBit, value, false, false)
	 */
	private static final void fill(String[][] array, int indexBit, String value) {
		fill(array, indexBit, value, false, false);
	}
	
	/**
	 * For each array at the indices of $array that have $indexBit set (or not set if $invert),
	 * set each index that has $indexBit2 set (or not set if $invert) to reference $value
	 */
	private static final void fill(String[][] array, int indexBit, int indexBit2, String value, boolean invert, boolean invert2, boolean lazy, boolean lazy2) {
		for(int i = 0; i < array.length; i++) {
			if((!lazy && ((i & indexBit) == indexBit && !invert) || ((i & indexBit) == 0 && invert)) ||
				(lazy && ((i & indexBit) != 0 && !invert)        || ((i & indexBit) != indexBit && invert))) {
				fill(array[i] = Arrays.copyOf(array[i], array[i].length), indexBit2, value, invert2, lazy2);
			}
		}
	}
	
	/**
	 * Calls fill(array, indexBit, indexBit2, value, false, false, false, false)
	 */
	private static final void fill(String[][] array, int indexBit, int indexBit2, String value) {
		fill(array, indexBit, indexBit2, value, false, false, false, false);
	}
	
	// help text
	private static final String[/* contexts */][/* buttons */][/* modifiers */][/* other */] statusText = new String[2][8][8][4];
	static {
		fill(statusText[background][left], "Go back");
		fill(statusText[background][left], control | shift, "Go forward", false, true);
		fill(statusText[background][left], drag, null); // no action for dragging unless shift down
		fill(statusText[background][left], shift | drag, panning, "Drag to pan");
		fill(statusText[background][middle], "Create arrow");
		fill(statusText[background][middle], drag, null); // no action for dragging
		fill(statusText[background][right], "Create rectangle");
		fill(statusText[background][right], drag, null); // no action for dragging
		fill(statusText[background][left | middle], null); // no action for left + middle
		fill(statusText[background][left | right], "Auto format");
		fill(statusText[background][middle | right], "Undo");
    	
    	fill(statusText[item][left], "Follow link");
    	fill(statusText[item][left], control, action, "Run action", true, false, false, false);
    	fill(statusText[item][left], drag, null); // no action for dragging
    	fill(statusText[item][middle], "Pickup");
    	fill(statusText[item][middle], drag, null); // no action for dragging
    	fill(statusText[item][right], "Copy");
    	fill(statusText[item][right], drag, null); // no action for dragging
    	fill(statusText[item][left | middle], null); // no action for left + middle
		fill(statusText[item][left | right], "Extract attributes");
		fill(statusText[item][middle | right], "Delete");
	}
	
	private static final String commandPadding = "     ";
	private static final String padding = "                  ";
	
	private static final String command(int mod, int button) {
		String cmd = modifiers[mod] + buttons[button];
		return commandPadding.substring(cmd.length()) + cmd + separator;
	}
	
	public static void updateStatus() {
		Item current = FrameUtils.getCurrentItem();
		int context = current != null ? item : 0;
		int mod = (FrameMouseActions.isControlDown() ? control : 0) | (FrameMouseActions.isShiftDown() ? shift : 0);
		int other = (ExperimentalFeatures.MousePan.get() ? panning : 0) | (current != null ? current.hasAction() ? action : 0 : 0);
		String status = "";
		
		String l = statusText[context][left][mod][other];
		String m = statusText[context][middle][mod][other];
		String r = statusText[context][right][mod][other];
		String lr = statusText[context][left | right][mod][other];
		String lm = statusText[context][left | middle][mod][other];
		String mr = statusText[context][middle | right][mod][other];
		String dl = statusText[context][left][mod | drag][other];
		String dm = statusText[context][middle][mod | drag][other];
		String dr = statusText[context][right][mod | drag][other];
		
		if(l != null)  status += command(mod, left) + l + padding.substring(l.length());
		if(m != null)  status += command(mod, middle) + m + padding.substring(m.length());
		if(r != null)  status += command(mod, right) + r + padding.substring(r.length());
		if(lm != null) status += command(mod, left | middle) + lm + padding.substring(lm.length());
		if(lr != null) status += command(mod, left | right) + lr + padding.substring(lr.length());
		if(mr != null) status += command(mod, middle | right) + mr + padding.substring(mr.length());
		if(dl != null || dm != null || dr != null) status += "\n";
		if(dl != null) status += command(mod | drag, left) + dl + padding.substring(dl.length());
		if(dm != null) status += command(mod | drag, left) + dm + padding.substring(dm.length());
		if(dr != null) status += command(mod | drag, left) + dr + padding.substring(dr.length());
		
		MessageBay.setStatus(status);
	}

}
